-- Created by 0x0ade for Marlin.

local switch = require("cs.celeste.switch")

-- This table will hold all hooks which will be applied later.
local switchHooks = {}

-- This value is checked in the FinishedCheck hook.
-- Switch.Activate runs FinishedCheck, which we want to avoid.
local finishedCheckSkip = false

-- Helper function determining if the given switch is permanent or not.
local function isPermanent(switch)
  local session = switch.scene.session
  local sid = session.area.sid
  local room = session.levelData.name

  -- Replace this with your/map/1-path and room-name
  return
    (sid == "insanelynicemap" and room == "1") or
    false
end

-- Helper function to get the save "flag" ID for the given switch.
local function getFlagID(switch)
  -- The switch itself is a component attached to a TouchSwitch,
  -- which is why we need to use the entity's position.
  local pos = switch.entity.position
  return string.format("switch_%d_%d", pos.x, pos.y)
end

function switchHooks:entityAdded(orig, scene)
  -- Run the original function code before running our own code.
  orig(self, scene)

  -- Activate the switch if the save flag for this switch has been set to true.
  if isPermanent(self) and scene.session:getFlag(getFlagID(self)) then
    finishedCheckSkip = true
    self:activate()
    finishedCheckSkip = false
  end
end

function switchHooks:activate(orig)
  -- Run the original function code and keep track of the return value.
  local finished = orig(self)

  -- If this switch is permanent, set the save flag for this switch to true.
  if isPermanent(self) then
    self.scene.session:setFlag(getFlagID(self), true)
  end

  -- We don't want to change the original return value.
  return finished
end

function switchHooks.finishedCheck(orig, level)
  -- If we're activating the switch while it's being added,
  -- don't check if all switches in the scene are active.
  -- It'd cause the check to return true for the first switch.
  if finishedCheckSkip then
    return false
  end

  return orig(level)
end

-- Apply all hooks at once.
for k, v in pairs(switchHooks) do
  switch[k]:hook(v)
end
